@@ -14,4 +14,12 @@
 
 #undef HAVE_FREETYPE
 
+#undef HAVE_STD_HASHMAP
+
+#undef HAVE_GNU_CXX_HASHMAP
+
+#undef HAVE_STD_SLIST
+
+#undef HAVE_GNU_CXX_SLIST
+
 #endif
