@@ -1,9 +1,32 @@
+/*
+  libuta - a C++ widget library based on SDL (Simple Direct Layer)
+  Copyright (C) 1999-2002  Karsten Laux <klaux@student.uni-kl.de>
+
+  This library is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
+  
+  This library is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Lesser General Public License for more details.
+  
+  You should have received a copy of the GNU Lesser General Public
+  License along with this library; if not, write to the
+  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+  Boston, MA  02111-1307, SA.
+*/
+
 #include "blitters.h"
-#include <uta/painter.h>
+#include <painter.h>
 
 namespace uta {
 
-#ifndef X86_ASSEMBLER
+#ifdef __OpenBSD__
+#undef X86_ASSEMBLER
+#endif
+#if !defined(X86_ASSEMBLER)
 
 /** Apply const gamma value.
     Blit just copies alpha value of the source to the destination surface.
@@ -131,7 +154,7 @@ void gammablit_RGBA8888_RGBA8888(Surface* dst, const Surface* src,
 	  if(x_misalign)
 	    {
 	      // read 4 bytes (1 pixel) from source memory -> mm2 
-	      movd_m2r((long)sPtr, mm1);
+	      movd_m2r(*(mmx_t*)sPtr, mm1);
 	      
 	      // add 8 bytes with unsigned saturation
 	      paddusb_r2r(mm0,mm1);  
@@ -179,7 +202,7 @@ void gammablit_RGBA8888_RGBA8888(Surface* dst, const Surface* src,
 	  if(x_misalign)
 	    {
 	      // read 4 bytes (1 pixel) from source memory -> mm2 
-	      movd_m2r((long)sPtr, mm1);
+	      movd_m2r(*(mmx_t*)sPtr, mm1);
 	      
 	      // substract 8 bytes with unsigned saturation
 	      psubusb_r2r(mm0,mm1);  
@@ -614,6 +637,7 @@ void gammablit_RGB565_RGB565(Surface* dst, const Surface* src,
   unsigned dstSkip = dst->pitch() - srcRect.width() * dst->pixelformat().bpp();
   unsigned srcSkip = src->pitch() - srcRect.width() * src->pixelformat().bpp();
   unsigned x_reload = srcRect.width()*src->pixelformat().bpp() / 8;
+  // misalign : 0,1,2 or 3 pixels !
   unsigned x_misalign = srcRect.width()*src->pixelformat().bpp() % 8;  
 
   mmx_t gamma_val;
@@ -633,8 +657,10 @@ void gammablit_RGB565_RGB565(Surface* dst, const Surface* src,
   
   mmx_t h;
   
-  /* hmm ... should we care for (double) word alignment when accessing memory ? 
+  /* hmm ... should we care for (double) word alignment 
+     when accessing memory ? 
      ... will need to investigate  */
+
   if(brighten)
     {
       y = srcRect.height();
@@ -643,7 +669,7 @@ void gammablit_RGB565_RGB565(Surface* dst, const Surface* src,
 	  if(x_misalign % 2)
 	    {
 	      // read 4 bytes (2 pixel) from source memory -> mm2 
-	      movd_m2r((long)sPtr, mm2);
+	      movd_m2r(*(mmx_t*)sPtr, mm2);
 	      
 	      // unpack mm2 to mm2,mm3,mm4
 	      UNPACK_32_RGB565_RGB888;
@@ -656,7 +682,7 @@ void gammablit_RGB565_RGB565(Surface* dst, const Surface* src,
 	      // pack components mm2,mm3,mm4 to mm2
 	      PACK_32_RGB888_RGB565;
 	      
-	      /* finally write back the results (32bits),
+	      /* finally write back the result,
 		 only using the lower 16bits of result
 		 ... this is somehow dumb, but only executed once
 		     per scanline, so I do not care */
@@ -665,9 +691,9 @@ void gammablit_RGB565_RGB565(Surface* dst, const Surface* src,
 	      sPtr += 2;
 	      dPtr += 2;
 	    }
-	  
-	  x = x_misalign / 2;
-	  while(x--)
+	  	  
+	  // now for the two remaining misaligned pixels
+	  if(x_misalign / 2)
 	    {
 	      // read 4 bytes (2 pixel) from source memory -> mm2 
 	      movd_m2r(*(mmx_t*)sPtr, mm2);
@@ -683,11 +709,12 @@ void gammablit_RGB565_RGB565(Surface* dst, const Surface* src,
 	      // pack components mm2,mm3,mm4 to mm2
 	      PACK_32_RGB888_RGB565;
 	      
-	      // finally write back the results (32 bits)
+	      // finally write back the results (dword: 32 bits)
 	      movd_r2m(mm2,*(mmx_t*)dPtr);
 	      
 	      sPtr+=4;
 	      dPtr+=4;
+	
 	    }
 	  
 	  x = x_reload;
@@ -735,7 +762,7 @@ void gammablit_RGB565_RGB565(Surface* dst, const Surface* src,
 	      // unpack mm2 to mm2,mm3,mm4
 	      UNPACK_32_RGB565_RGB888;
 	      
-	      // add 8 bytes with unsigned saturation
+	      // sub 8 bytes with unsigned saturation
 	      psubusb_r2r(mm0,mm2);  
 	      psubusb_r2r(mm0,mm3);  
 	      psubusb_r2r(mm0,mm4);  
@@ -753,16 +780,17 @@ void gammablit_RGB565_RGB565(Surface* dst, const Surface* src,
 	      dPtr += 2;
 	    }
 	  
-	  x = x_misalign / 2;
-	  while(x--)
-	    {
+	
+	  // now for the two remaining misaligned pixels
+	  if(x_misalign / 2)
+	  {
 	      // read 4 bytes (2 pixel) from source memory -> mm2 
 	      movd_m2r(*(mmx_t*)sPtr, mm2);
 	      
 	      // unpack mm2 to mm2,mm3,mm4
 	      UNPACK_32_RGB565_RGB888;
 	      
-	      // add 8 bytes with unsigned saturation
+	      // sub 8 bytes with unsigned saturation
 	      psubusb_r2r(mm0,mm2);  
 	      psubusb_r2r(mm0,mm3);  
 	      psubusb_r2r(mm0,mm4);  
@@ -786,7 +814,7 @@ void gammablit_RGB565_RGB565(Surface* dst, const Surface* src,
 	      
 	      UNPACK_64_RGB565_RGB888;
 	      
-	      // add 8 bytes with unsigned saturation
+	      // sub 8 bytes with unsigned saturation
 	      psubusb_r2r(mm0,mm2);  
 	      psubusb_r2r(mm0,mm3);  
 	      psubusb_r2r(mm0,mm4);  
@@ -916,7 +944,7 @@ void gammablit_RGBA8888_RGBA8888(Surface* dst, const Surface* src,
 	  por_r2r(mm1,mm0);  // 1,2,3,5,6,7
 
 	  // read 4 bytes (1 pixel) from source memory -> mm2 
-	  movd_m2r((long)sPtr, mm1);
+	  movd_m2r(*(mmx_t*)sPtr, mm1);
 
 	  if(g < 0)
 	    { 
@@ -1070,7 +1098,7 @@ void gammablit_RGBA8888_RGB565(Surface* dst, const Surface* src,
       if(x_misalign)
 	{
 	  // read 4 bytes (1 pixel) from source memory -> mm2 
-	  movd_m2r((long)sPtr, mm2);
+	  movd_m2r(*(mmx_t*)sPtr, mm2);
 	  
 	  /* and copy the value to mm3 and mm4 */ 
 	  movq_r2r(mm2, mm3); 
@@ -1294,12 +1322,13 @@ void gammablit_RGB565_RGB565(Surface* dst, const Surface* src,
   
   mmx_t h;
   
-  /* hmm ... should we care for (double) word alignment when accessing memory ? 
+  /* hmm ... should we care for (double) word alignment 
+     when accessing memory ? 
      ... will need to investigate  */
   
   y = srcRect.height();
   while(y--)
-    {
+    {     	
       if(x_misalign % 2)
 	{
 	  g = *gPtr;
@@ -1325,7 +1354,7 @@ void gammablit_RGB565_RGB565(Surface* dst, const Surface* src,
 	  por_r2r(mm1,mm0);  
 
 	  // read 4 bytes (2 pixel) from source memory -> mm2 
-	  movd_m2r((long)sPtr, mm2);
+	  movd_m2r(*(mmx_t*)sPtr, mm2);
 	  
 	  // unpack mm2 to mm2,mm3,mm4
 	  UNPACK_32_RGB565_RGB888;
@@ -1358,9 +1387,10 @@ void gammablit_RGB565_RGB565(Surface* dst, const Surface* src,
 	  dPtr += 2;
 	}
       
-      x = x_misalign / 2;
-      while(x--)
-	{
+     
+      // now for the two remaining misaligned pixels
+      if(x_misalign / 2)
+      {
 	  g = *gPtr;
 	  GAMMA_PTR_STEP();
 
@@ -1599,7 +1629,7 @@ void gammablit(Surface* dst, const Surface* src,
       gammablit_C(dst,src,g,srcRect,dstPoint);
     }
 #else
-  gammablit_C(dst,src,g,srcRect,dstPoint)
+  gammablit_C(dst,src,g,srcRect,dstPoint);
 #endif
 }
 
@@ -1626,7 +1656,7 @@ void gammablit(Surface* dst, const Surface* src,
       gammablit_C(dst,src,func,srcRect,dstPoint);
     }
 #else
-  gammablit_C(dst,src,func,srcRect,dstPoint)
+  gammablit_C(dst,src,func,srcRect,dstPoint);
 #endif
 }
 
