@@ -3,9 +3,9 @@
 //  Copyright (C) 1998,1999  The Worldforge Project
 //
 //  This program is free software; you can redistribute it and/or modify
-//  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version.
+//  it under the terms of the GNU Lesser General Public License as 
+//  published by the Free Software Foundation; either version 2.1 of the
+//  License, or (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +24,7 @@
 
 namespace uta {
 
-unsigned long debug_mask;
+unsigned long debug_mask = 0;
 
 }
 
