@@ -1,18 +1,18 @@
 /*
-  libuta - a C++ widget library based on SDL (Simple Direct Layer)
-  Copyright (C) 1999  Karsten Laux
+  libuta - a C++ widget library based on SDL (Simple DirectMedia Layer)
+  Copyright (C) 1999-2002  Karsten Laux
 
   This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Library General Public
+  modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
-  version 2 of the License, or (at your option) any later version.
+  version 2.1 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Library General Public License for more details.
+  Lesser General Public License for more details.
   
-  You should have received a copy of the GNU Library General Public
+  You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, SA.
@@ -151,7 +151,7 @@ Surface::createChildSurface(const Rect& rect)
 }
 
 void 
-Surface::setPalette(const vector<Color>& pal)
+Surface::setPalette(const std::vector<Color>& pal)
 {  
   
   format.palette = pal;
@@ -197,7 +197,7 @@ Surface::applyPalette()
 	{
 	  if(!SDL_SetColors(sdlSurface_,colors,0,count))
 	    {
-	      debugN(17, cerr << "Surface: unable to set palette properly. "<< endl;);
+	      debugN(17, std::cerr << "Surface: unable to set palette properly. "<< std::endl;);
 	    }
 	}
   
@@ -353,9 +353,9 @@ Surface::lock() const
 	  SDL_Delay(10);
 	  if(SDL_LockSurface(sdlSurface_) < 0)
 	    {
-	      cerr << "Surface::lock on surface failed twice." 
-		   << endl;
-	      cerr << "         no handling here yet :-(" << endl;
+	      std::cerr << "Surface::lock on surface failed twice." 
+		   << std::endl;
+	      std::cerr << "         no handling here yet :-(" << std::endl;
 	      assert(false);
 	    }
 	}
@@ -438,7 +438,320 @@ Surface::readPixel(Uint32 pixeladdr)
   return pixel;
 }
 
+// Calculate the shift needed to make the most
+// significant bit of an unsigned char line up
+// with the most significant bit in the mask
+static int g_shift(Uint32 mask, int shift)
+{
+  switch(mask >> shift) {
+   case 0x1:
+     return shift - 7;
+   case 0x3:
+     return shift - 6;
+   case 0x7:
+     return shift - 5;
+   case 0xf:
+     return shift - 4;
+   case 0x1f:
+     return shift - 3;
+   case 0x3f:
+     return shift - 2;
+   case 0x7f:
+     return shift - 1;
+   case 0xff:
+     return shift;
+   default:
+     assert(false);
+  }
+}
 
+// Lighten (g > 0) or darken (g < 0) the shade of
+// all pixels in a surface. The value of g is added
+// to each of the (red,green,blue) color values,
+// where the colors are treated as if they were
+// eight bit values (i.e. adding 127 will take black
+// to grey and grey to white)
+void
+Surface::lighten_darken(char g)
+{
+  if(g == 0 || !sdlSurface_)
+    return;
+
+  const SDL_PixelFormat *format = sdlSurface_->format;
+
+  assert(format);
+
+  int bytespp = format->BytesPerPixel;
+  assert(bytespp == 2 || bytespp == 3 || bytespp == 4);
+
+  int pitch = sdlSurface_->pitch;
+
+  // deal with 3 bytes per pixel as a special case to avoid
+  // alignment problems
+  if(bytespp == 3) {
+    // Assume one byte per color
+    assert(format->Rmask >> format->Rshift == 0xff);
+    assert(format->Gmask >> format->Gshift == 0xff);
+    assert(format->Bmask >> format->Bshift == 0xff);
+
+    lock();
+
+    unsigned char* pixels = (unsigned char*) sdlSurface_->pixels;
+
+    for(int x = 0; x < sdlSurface_->w * 3; ++x) {
+      for(int y = 0; y < sdlSurface_->h; ++y) {
+        unsigned char *pixelpos = pixels + x + y * pitch;
+
+        if(g > 0) {
+          Uint16 tmp = ((Uint16) g ) + *pixelpos;
+          *pixelpos = (tmp > 0xff) ? 0xff : tmp;
+        }
+        else {
+          *pixelpos = (*pixelpos > -g) ? *pixelpos + g : 0;
+        }
+      }
+    }
+
+    unlock();
+    return;
+  }
+
+  Uint32 Rmask = format->Rmask;
+  Uint32 Gmask = format->Gmask;
+  Uint32 Bmask = format->Bmask;
+  Uint32 Amask = format->Amask;
+
+  // Instead of shifting the red bits in the pixel down to get
+  // an eight bit value, adding, and shifting up again, we
+  // shift g up so its most significant bit lines up with the
+  // most significant bit in the red mask. Same for green, blue.
+  // Since we need g > 0 to pull this trick, for the case g < 0
+  // we shift -g to get a similar effect.
+  
+  Uint32 Rgam, Ggam, Bgam;
+
+  if(g > 0) {
+    Rgam = (((Uint32) g) << g_shift(Rmask, format->Rshift)) & Rmask;
+    Ggam = (((Uint32) g) << g_shift(Gmask, format->Gshift)) & Gmask;
+    Bgam = (((Uint32) g) << g_shift(Bmask, format->Bshift)) & Bmask;
+  }
+  else {
+    Rgam = (((Uint32) (-g)) << g_shift(Rmask, format->Rshift)) & Rmask;
+    Ggam = (((Uint32) (-g)) << g_shift(Gmask, format->Gshift)) & Gmask;
+    Bgam = (((Uint32) (-g)) << g_shift(Bmask, format->Bshift)) & Bmask;
+  }
+
+  lock();
+
+  unsigned char* pixels = (unsigned char*) sdlSurface_->pixels;
+
+  // Walk through the surface, add g for each color
+  for(Uint16 x = 0; x < sdlSurface_->w; ++x) {
+    for(Uint16 y = 0; y < sdlSurface_->h; ++y) {
+      unsigned char* pixelpos = pixels + x * bytespp + y * pitch;
+
+      Uint32 pixel;
+
+      if(bytespp == 2)
+        pixel = *((const Uint16*) pixelpos);
+      else // bytespp == 4
+        pixel = *((const Uint32*) pixelpos);
+
+      Uint32 red, green, blue, tmp;
+
+      if(g > 0) {
+        tmp = (pixel & Rmask);
+        red = (Rmask - tmp > Rgam) ? Rgam + tmp : Rmask;
+
+        tmp = (pixel & Gmask);
+        green = (Gmask - tmp > Ggam) ? Ggam + tmp : Gmask;
+
+        tmp = (pixel & Bmask);
+        blue = (Bmask - tmp > Bgam) ? Bgam + tmp : Bmask;
+      }
+      else {
+        tmp = pixel & Rmask;
+        red = (tmp > Rgam) ? tmp - Rgam : 0;
+
+        tmp = pixel & Gmask;
+        green = (tmp > Ggam) ? tmp - Ggam : 0;
+
+        tmp = pixel & Bmask;
+        blue = (tmp > Bgam) ? tmp - Bgam : 0;
+      }
+
+      Uint32 outpixel = red | green | blue | (pixel & Amask);
+
+      if(bytespp == 2)
+        *((Uint16*) pixelpos) = outpixel;
+      else // bytespp == 4
+        *((Uint32*) pixelpos) = outpixel;
+    }
+  }
+
+  unlock();
+}
+
+// An uta::GammaFunction acts like a texture which
+// shades the target surface differentlt at different points.
+// It's a 256x256 tile. The (x >> 2) and (y >> 1) are there
+// to emulate a hack of Karsten's in the old (deprecated)
+// blitter code.
+static inline char get_gamma(const GammaFunction *gamma, int x, int y)
+{
+  // Bitwise & with 0xff gives (mod 256). It's also
+  // safe for negative values of the offset.
+  return gamma->values[((gamma->offset_x + (x >> 2)) & 0xff)
+    + (((gamma->offset_y + (y >> 1)) & 0xff) << 8)];
+}
+
+// This works the same as the other lighten_darken() function,
+// except we use a texture instead of a constant
+// value for g.
+void
+Surface::lighten_darken(const GammaFunction *gamma)
+{
+  if(!gamma || !sdlSurface_)
+    return;
+
+  const SDL_PixelFormat *format = sdlSurface_->format;
+
+  assert(format);
+
+  int bytespp = format->BytesPerPixel;
+  assert(bytespp == 2 || bytespp == 3 || bytespp == 4);
+
+  int pitch = sdlSurface_->pitch;
+
+  // deal with 3 bytes per pixel as a special case to avoid
+  // alignment problems
+  if(bytespp == 3) {
+    // Assume one byte per color
+    assert(format->Rmask >> format->Rshift == 0xff);
+    assert(format->Gmask >> format->Gshift == 0xff);
+    assert(format->Bmask >> format->Bshift == 0xff);
+
+    lock();
+
+    unsigned char* pixels = (unsigned char*) sdlSurface_->pixels;
+
+    for(int x = 0; x < sdlSurface_->w; ++x) {
+      for(int y = 0; y < sdlSurface_->h; ++y) {
+        Uint16 g = get_gamma(gamma, x, y);
+        if(g == 0)
+          continue;
+
+        for(int i = 0; i < 3; ++i) {
+          unsigned char *pixelpos = pixels + i + x * 3 + y * pitch;
+
+          if(g > 0) {
+            Uint16 tmp = g + *pixelpos;
+            *pixelpos = (tmp > 0xff) ? 0xff : tmp;
+          }
+          else {
+            *pixelpos = (*pixelpos > -g) ? *pixelpos + g : 0;
+          }
+        }
+      }
+    }
+
+    unlock();
+    return;
+  }
+
+  Uint32 Rmask = format->Rmask;
+  Uint32 Gmask = format->Gmask;
+  Uint32 Bmask = format->Bmask;
+  Uint32 Amask = format->Amask;
+
+  int RmodShift = g_shift(Rmask, format->Rshift);
+  int GmodShift = g_shift(Gmask, format->Gshift);
+  int BmodShift = g_shift(Bmask, format->Bshift);
+
+  // Start Rgam, Ggam, Bgam with values that match the
+  // starting value of old_g
+  Uint32 Rgam = 0, Ggam = 0, Bgam = 0;
+  char old_g = 0;
+
+  lock();
+
+  unsigned char* pixels = (unsigned char*) sdlSurface_->pixels;
+
+  for(Uint16 x = 0; x < sdlSurface_->w; ++x) {
+    for(Uint16 y = 0; y < sdlSurface_->h; ++y) {
+
+      char g = get_gamma(gamma, x, y);
+      if(g == 0)
+        continue;
+
+      if(g != old_g) { // only recompute if we need to
+
+        // Instead of shifting the red bits in the pixel down to get
+        // an eight bit value, adding, and shifting up again, we
+        // shift g up so its most significant bit lines up with the
+        // most significant bit in the red mask. Same for green, blue.
+        // Since we need g > 0 to pull this trick, for the case g < 0
+        // we shift -g to get a similar effect.
+
+        if(g > 0) {
+          Rgam = (((Uint32) g) << RmodShift) & Rmask;
+          Ggam = (((Uint32) g) << GmodShift) & Gmask;
+          Bgam = (((Uint32) g) << BmodShift) & Bmask;
+        }
+        else {
+          Rgam = (((Uint32) (-g)) << RmodShift) & Rmask;
+          Ggam = (((Uint32) (-g)) << GmodShift) & Gmask;
+          Bgam = (((Uint32) (-g)) << BmodShift) & Bmask;
+        }
+
+        old_g = g;
+      }
+
+      unsigned char* pixelpos = pixels + x * bytespp + y * pitch;
+
+      Uint32 pixel;
+
+      if(bytespp == 2)
+        pixel = *((const Uint16*) pixelpos);
+      else // bytespp == 4
+        pixel = *((const Uint32*) pixelpos);
+
+      Uint32 red, green, blue, tmp;
+
+      if(g > 0) {
+        tmp = (pixel & Rmask);
+        red = (Rmask - tmp > Rgam) ? Rgam + tmp : Rmask;
+
+        tmp = (pixel & Gmask);
+        green = (Gmask - tmp > Ggam) ? Ggam + tmp : Gmask;
+
+        tmp = (pixel & Bmask);
+        blue = (Bmask - tmp > Bgam) ? Bgam + tmp : Bmask;
+      }
+      else {
+        Uint32 tmp;
+
+        tmp = pixel & Rmask;
+        red = (tmp > Rgam) ? tmp - Rgam : 0;
+
+        tmp = pixel & Gmask;
+        green = (tmp > Ggam) ? tmp - Ggam : 0;
+
+        tmp = pixel & Bmask;
+        blue = (tmp > Bgam) ? tmp - Bgam : 0;
+      }
+
+      Uint32 outpixel = red | green | blue | (pixel & Amask);
+
+      if(bytespp == 2)
+        *((Uint16*) pixelpos) = outpixel;
+      else // bytespp == 4
+        *((Uint32*) pixelpos) = outpixel;
+    }
+  }
+
+  unlock();
+}
 
 Rect
 Surface::blit(Surface* destBM) const
@@ -496,7 +809,7 @@ Surface::blit(Surface* destBM, const Rect& dest, const Rect& src_) const
   src.w = src_.width();
   src.h = src_.height();  
                 
-  //  debugN(17,cerr<<src_<<">>"<< dest<<endl;);
+  //  debugN(17,std::cerr<<src_<<">>"<< dest<<std::endl;);
 
   SDL_BlitSurface(sdlSurface_, &src, destBM->sdlSurface_, &dst);
   
@@ -530,13 +843,13 @@ Surface::fill(const Rect &dest_, const Color &color)
 {
    if(sdlSurface_ == 0)
      {
-       debugN(17, cerr << "Surface.fill() on empty Surface has no effect." << endl;);
+       debugN(17, std::cerr << "Surface.fill() on empty Surface has no effect." << std::endl;);
        return;
      }
 
    if(!dest_.isValid())
      {
-       debugN(17, cerr << "Surface.fill() was given an invalid rect." << endl;);
+       debugN(17, std::cerr << "Surface.fill() was given an invalid rect." << std::endl;);
        return;
      }
    
@@ -561,8 +874,8 @@ bool
 Surface::convert(const Pixelformat& pixelformat, bool _dither)
 {
 
-  debugN(17,cerr << "Surface converting "<< format.asString() 
-	<<" to " << pixelformat.asString() << " ... "<< endl;);
+  debugN(17,std::cerr << "Surface converting "<< format.asString() 
+	<<" to " << pixelformat.asString() << " ... "<< std::endl;);
 
   bool success = false;
 
@@ -632,9 +945,9 @@ Surface::convert(const Pixelformat& pixelformat, bool _dither)
 		  int b, b0, b1, b2;
 		  int* e = errors;
 		  
-		  r = r0 = r1 = 0;
-		  g = g0 = g1 = 0;
-		  b = b0 = b1 = 0;  
+		  r = r0 = r1 = r2 = 0;
+		  g = g0 = g1 = g2 = 0;
+		  b = b0 = b1 = b2 = 0;  
 		  
 		  for(int x = 0; x < width() ; x++)
 		    {      
@@ -739,17 +1052,17 @@ Surface::convert(const Pixelformat& pixelformat, bool _dither)
 	    }
 	  else
 	    {
-	      cerr << "Surface: out of memory." << endl;
+	      std::cerr << "Surface: out of memory." << std::endl;
 	    }
 	}
 
     }
   else
     {
-      debugN(17, cerr << "Surface: convert to unknown Pixelformat requested." << endl;);
+      debugN(17, std::cerr << "Surface: convert to unknown Pixelformat requested." << std::endl;);
     }
 
-  debugN(17,cerr << "Surface.convert() done." << endl;);
+  debugN(17,std::cerr << "Surface.convert() done." << std::endl;);
 
   return success;
     
@@ -838,7 +1151,7 @@ Surface::scaledBlit(Surface* dst, const Rect& dest, const Rect& src) const
 bool 
 Surface::scale(float n)
 {
-  return scale(int(width()*n), int(height()*n));
+  return scale((int)(width()*n + 0.5), (int)(height()*n + 0.5));
 }
 
 
