@@ -1,19 +1,19 @@
 /*
   libuta - a C++ widget library based on SDL (Simple Direct Layer)
 
-  Copyright (C) 1999  Karsten Laux <klaux@student.uni-kl.de>
+  Copyright (C) 1999-2002  Karsten Laux <klaux@student.uni-kl.de>
 
   This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Library General Public
+  modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
-  version 2 of the License, or (at your option) any later version.
+  version 2.1 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Library General Public License for more details.
+  Lesser General Public License for more details.
   
-  You should have received a copy of the GNU Library General Public
+  You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, SA.
@@ -21,9 +21,9 @@
 
   The read_png() routine has been copied from SDL_image-0.9.tar.gz
       IMGLIB:  An example image loading library for use with SDL
-      Copyright (C) 1999  Sam Lantinga
+      Copyright (C) 1999-2002  Sam Lantinga
       Changes: 
-        - replace IMG_error by cerr
+        - replace IMG_error by std::cerr
   
   The save_png() routine was originally created by  
   Philippe Lavoie <lavoie@zeus.genie.uottawa.ca> on 2 November 1998
@@ -41,16 +41,16 @@
           Copyright (C) 1998,1999 Philippe Lavoie
  
           This library is free software; you can redistribute it and/or
-          modify it under the terms of the GNU Library General Public
+          modify it under the terms of the GNU Lesser General Public
           License as published by the Free Software Foundation; either
-          version 2 of the License, or (at your option) any later version.
+          version 2.1 of the License, or (at your option) any later version.
  
           This library is distributed in the hope that it will be useful,
           but WITHOUT ANY WARRANTY; without even the implied warranty of
           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-          Library General Public License for more details.
+          Lesser General Public License for more details.
  
-          You should have received a copy of the GNU Library General Public
+          You should have received a copy of the GNU Lesser General Public
           License along with this library; if not, write to the Free
           Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, SA.
 
@@ -61,7 +61,7 @@
 
 
   Modifications by Karsten Laux:
-  * replaced SDL_SetError by cerr 
+  * replaced SDL_SetError by std::cerr 
   * filehandling and writing merged to one function
   * moved vars declaration beneath setjmp to get rid of compiler warnings
 */
@@ -70,7 +70,7 @@
 #include "surface.h"
 #include "painter.h"
 #include "color.h"
-#
+ 
 #include "debug.h"
 
 #include <iostream>
@@ -78,6 +78,7 @@
 #include <cstdlib>
 #include <cassert>
 #include <cctype>
+#include <map>
 
 #include <png.h>
 #include <SDL/SDL_endian.h>
@@ -101,13 +102,13 @@ SDL_Surface* read_png(const char* filename)
   fp = fopen(filename, "rb");
   if( fp == 0)
     {
-      debugN(17,cerr << "     ! Couldn't open " << filename << " for reading." 
-	     << endl;);
+      debugN(17,std::cerr << "     ! Couldn't open " << filename << " for reading." 
+	     << std::endl;);
       return 0;
     }
   else
     {
-      debugN(17,cerr << "     * reading file " << filename << endl;);
+      debugN(17,std::cerr << "     * reading file " << filename << std::endl;);
     }
 
   png_structp png_ptr;
@@ -123,7 +124,7 @@ SDL_Surface* read_png(const char* filename)
   Uint32 Bmask;
   Uint32 Amask;
   SDL_Palette *palette;
-  png_bytep *row_pointers;
+  png_bytep *row_pointers = NULL;
   unsigned int row, i;
   int ckey = -1;
   png_color_16 *transv;
@@ -133,7 +134,7 @@ SDL_Surface* read_png(const char* filename)
 				   0,0,0);
 
   if (png_ptr == 0){
-    cerr << "     ! Couldn't allocate the memory for a PNG info structs." << endl;
+    std::cerr << "     ! Couldn't allocate the memory for a PNG info structs." << std::endl;
     return 0;
   }
   
@@ -141,14 +142,14 @@ SDL_Surface* read_png(const char* filename)
   info_ptr = png_create_info_struct(png_ptr);
   if (info_ptr == 0)
     {
-      cerr << "      ! Couldn't create the image information for a PNG file" << endl;
+      std::cerr << "      ! Couldn't create the image information for a PNG file" << std::endl;
       png_destroy_read_struct(&png_ptr, (png_infopp)0, (png_infopp)0);
       return 0;
     }
   end_info = png_create_info_struct(png_ptr);
   if (end_info == 0)
     {
-      cerr << "     ! Couldn't create the image information for a PNG file" << endl;
+      std::cerr << "     ! Couldn't create the image information for a PNG file" << std::endl;
       png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)0);
       return 0;
     }
@@ -161,7 +162,7 @@ SDL_Surface* read_png(const char* filename)
     {
       /* Free all of the memory associated with the png_ptr and info_ptr */
       png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)0);
-      cerr << "     ! Error reading the PNG file."<< endl;
+      std::cerr << "     ! Error reading the PNG file."<< std::endl;
       /* If we get here, we had a problem reading the file */
       return 0;
    }
@@ -250,7 +251,7 @@ SDL_Surface* read_png(const char* filename)
   
   if ( surface == NULL ) 
     {
-      cerr << "      ! out of memory"<<endl;
+      std::cerr << "      ! out of memory"<<std::endl;
       goto done;
     }
   
@@ -266,7 +267,7 @@ SDL_Surface* read_png(const char* filename)
   row_pointers = (png_bytep*) malloc(sizeof(png_bytep)*height);
   if ( (row_pointers == NULL) ) 
     {
-      cerr << "      ! out of memory !" << endl;
+      std::cerr << "      ! out of memory !" << std::endl;
       SDL_FreeSurface(surface);
       surface = NULL;
       goto done;
@@ -366,12 +367,12 @@ SDL_Surface* read_png(const char* filename)
 //   }
   
     
-//     debugN(17,cerr<<"     * allocating surface "<<width<<" x "<<height<<" pixels"<< endl;);
+//     debugN(17,std::cerr<<"     * allocating surface "<<width<<" x "<<height<<" pixels"<< std::endl;);
 
 //   /* Create a 32bit surface, note that the colors are RGB ordered */
 //   if(hasAlpha)
 //     {
-//       debugN(17,cerr<<"    * creating alpha-blit capable surface."<<endl;);
+//       debugN(17,std::cerr<<"    * creating alpha-blit capable surface."<<std::endl;);
 //       surface = SDL_AllocSurface(SDL_SWSURFACE |SDL_SRCALPHA,
 // 				 width, 
 // 				 height, 
@@ -380,7 +381,7 @@ SDL_Surface* read_png(const char* filename)
 //     }
 //   else
 //     {
-//       debugN(17,cerr<<"    * creating normal surface."<<endl;);
+//       debugN(17,std::cerr<<"    * creating normal surface."<<std::endl;);
 //       surface = SDL_AllocSurface(SDL_SWSURFACE,
 // 				 width, 
 // 				 height, 
@@ -391,7 +392,7 @@ SDL_Surface* read_png(const char* filename)
 
 //   if ( surface == 0 ) 
 //     {
-//       cerr << "     ! failed to allocate surface memory." << endl;
+//       std::cerr << "     ! failed to allocate surface memory." << std::endl;
 //       return(0);
 //     }
 
@@ -434,7 +435,7 @@ int write_png(const char* filename, SDL_Surface *surface)
 
   fp = fopen(filename, "wb");
   if ( fp == 0 ) {
-    cerr << "Couldn't open " << filename << "for writing" << endl;
+    std::cerr << "Couldn't open " << filename << "for writing" << std::endl;
     return -1;
   }
 
@@ -444,7 +445,7 @@ int write_png(const char* filename, SDL_Surface *surface)
   png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,0,0,0);
   
   if (png_ptr == 0){
-    cerr << "Couldn't create a write structure for the PNG file." << endl;
+    std::cerr << "Couldn't create a write structure for the PNG file." << std::endl;
     fclose(fp);
     return -1;
   }
@@ -452,7 +453,7 @@ int write_png(const char* filename, SDL_Surface *surface)
   /* Allocate/initialize the image information data.  REQUIRED */
   info_ptr = png_create_info_struct(png_ptr);
   if (info_ptr == 0){
-    cerr << "Couldn't create the image information data." << endl;
+    std::cerr << "Couldn't create the image information data." << std::endl;
     png_destroy_write_struct(&png_ptr,  (png_infopp)0);
     fclose(fp);
     return -1;
@@ -463,7 +464,7 @@ int write_png(const char* filename, SDL_Surface *surface)
    */
   if (setjmp(png_ptr->jmpbuf)){
     /* If we get here, we had a problem reading the file */
-    cerr << "Error reading the file." << endl;
+    std::cerr << "Error reading the file." << std::endl;
     png_destroy_write_struct(&png_ptr,  (png_infopp)0);
     fclose(fp);
     return -1;
@@ -507,7 +508,7 @@ int write_png(const char* filename, SDL_Surface *surface)
   }
   else{
     if(surface->format->BitsPerPixel != 32 ){
-      cerr << "The author of the PNG saver was lazy and doesn't support something other than 32bpp in non palette images." << endl;
+      std::cerr << "The author of the PNG saver was lazy and doesn't support something other than 32bpp in non palette images." << std::endl;
       fclose(fp);
       return -1 ; 
     }
@@ -644,14 +645,14 @@ int write_png(const char* filename, SDL_Surface *surface)
   
   if ( fclose(fp) == EOF ) 
     {
-      cerr << "Couldn't close the file %s" << endl;
+      std::cerr << "Couldn't close the file %s" << std::endl;
       retval = -1;
     }
   
   return retval;
 }
 
-bool Surface::writeToFile(const string& filename)
+bool Surface::writeToFile(const std::string& filename)
 {
   
   bool success = false;
@@ -659,8 +660,8 @@ bool Surface::writeToFile(const string& filename)
   // set the global var
   pixelformat_ = format;
 
-  if(filename.find(".bmp") != string::npos ||
-     filename.find(".BMP") != string::npos)
+  if(filename.find(".bmp") != std::string::npos ||
+     filename.find(".BMP") != std::string::npos)
     {
       success = !SDL_SaveBMP(sdlSurface_, filename.c_str());
     }
@@ -671,22 +672,22 @@ bool Surface::writeToFile(const string& filename)
 
   if(success)
     {    
-      cerr << "Surface wrote \""<<filename<<"\" successfully." << endl;
+      std::cerr << "Surface wrote \""<<filename<<"\" successfully." << std::endl;
     }
   else
     {
-      cerr << "Surface could not write \""<<filename<<"\" to disk."  << endl;
+      std::cerr << "Surface could not write \""<<filename<<"\" to disk."  << std::endl;
     }
 
   return success;
 }
 
 
-bool Surface::readFromFile(const string& filename)
+bool Surface::readFromFile(const std::string& filename)
 {
   bool success = false;
 
-  debugN(17,cerr<<"Surface::readFromFile("<<filename<<")"<<endl;);
+  debugN(17,std::cerr<<"Surface::readFromFile("<<filename<<")"<<std::endl;);
 
   //free old surface data (if any)
   clearSurfaceData();
@@ -695,8 +696,8 @@ bool Surface::readFromFile(const string& filename)
   format = Pixelformat::UNKNOWN;
   int pixformat;
 
-  if(filename.find(".bmp") != string::npos ||
-     filename.find(".BMP") != string::npos)
+  if(filename.find(".bmp") != std::string::npos ||
+     filename.find(".BMP") != std::string::npos)
     {
       sdlSurface_ = SDL_LoadBMP(filename.c_str());
     }
@@ -708,8 +709,8 @@ bool Surface::readFromFile(const string& filename)
 
   if(sdlSurface_)
     {
-      debugN(17,cerr << "     * successfully read file \"" << filename 
-	    << "\""<<endl;);
+      debugN(17,std::cerr << "     * successfully read file \"" << filename 
+	    << "\""<<std::endl;);
       pixformat = Pixelformat::identify(sdlSurface_->format->BytesPerPixel,
 					sdlSurface_->format->Rmask,
 					sdlSurface_->format->Gmask,
@@ -720,22 +721,22 @@ bool Surface::readFromFile(const string& filename)
       if(pixformat == Pixelformat::UNKNOWN)
 	{
 	  debugN(17,
-		 cerr.setf(ios::hex);
-		 cerr << "     ! surface is in unidentified pixelformat !" 
-		 << endl;
-		 cerr << sdlSurface_->format->BytesPerPixel << "," 
+		 std::cerr.setf(std::ios::hex);
+		 std::cerr << "     ! surface is in unidentified pixelformat !" 
+		 << std::endl;
+		 std::cerr << sdlSurface_->format->BytesPerPixel << "," 
 		 << sdlSurface_->format->Rmask << "," 
 		 << sdlSurface_->format->Gmask << "," 
 		 << sdlSurface_->format->Bmask << "," 
-		 << sdlSurface_->format->Amask << endl;
-		 cerr.unsetf(ios::hex);
+		 << sdlSurface_->format->Amask << std::endl;
+		 std::cerr.unsetf(std::ios::hex);
 		 );
 	}
       else
 	{
 	  format = pixformat;
-	  debugN(17,cerr << "     * got known pixelformat:"
-		 << format.asString() << endl; );
+	  debugN(17,std::cerr << "     * got known pixelformat:"
+		 << format.asString() << std::endl; );
  
 	  gatherPalette();
 
@@ -772,7 +773,7 @@ bool Surface::readFromHeader(char* header_data, unsigned w, unsigned h)
   
   success = true;
   
-  debugN(17, cerr << "parsing header_data ..."<< endl;);
+  debugN(17, std::cerr << "parsing header_data ..."<< std::endl;);
   Painter* painter = new Painter(this);
   
   unsigned char pixel[3];
@@ -792,7 +793,7 @@ bool Surface::readFromHeader(char* header_data, unsigned w, unsigned h)
       }
   
   delete painter;
-  debugN(17,cerr<<"OK."<<endl;);
+  debugN(17,std::cerr<<"OK."<<std::endl;);
   
   return success;
 }
@@ -811,8 +812,8 @@ Surface::readFromXPM(char** data)
   int lineNr = 0;
   char* current = data[lineNr++];
 
-  string line;
-  line = string(current); 
+  std::string line;
+  line = std::string(current); 
   
   // parse the header line (using goold old sscanf)
   int w, h;
@@ -822,23 +823,23 @@ Surface::readFromXPM(char** data)
   
   if(num != 4)
     {
-      debugN(17,cerr<<"Surface::readFromXPM failed to parse header." << endl;);
+      debugN(17,std::cerr<<"Surface::readFromXPM failed to parse header." << std::endl;);
       return false;
     }
 
-  debugN(17,cerr<<"Surface::readFromXPM going to parse colortable with "
-	 << colorCount <<" entries ..." << endl;);
+  debugN(17,std::cerr<<"Surface::readFromXPM going to parse colortable with "
+	 << colorCount <<" entries ..." << std::endl;);
 
   // now go for the color table
-  map<string, Color> colors;
+  std::map<std::string, Color> colors;
   Color color;
-  string code;
+  std::string code;
   for(int i = 0; i < colorCount; i++)
     {
       current = data[lineNr++];
       
       // get the color code
-      code = string(current, cpp);
+      code = std::string(current, cpp);
       // skip color code
       current += cpp;
 
@@ -852,7 +853,7 @@ Surface::readFromXPM(char** data)
 	  // this is a "normal" colortable entry
 	  current++; // skip '#'
 
-	  debugN(17, cerr << current << "--> ";);
+	  debugN(17, std::cerr << current << "--> ";);
 
 	  // who needs sscanf ?
 	  color.r = hexDecode(*current++)*16 + hexDecode(*current++);
@@ -865,7 +866,7 @@ Surface::readFromXPM(char** data)
 	  color.a = 0;
 #endif
 
-	  debugN(17,cerr << "("<<(int)color.r<<","<<(int)color.g<<","<<(int)color.b<<")"<<endl;);
+	  debugN(17,std::cerr << "("<<(int)color.r<<","<<(int)color.g<<","<<(int)color.b<<")"<<std::endl;);
 	}
       else
 	{
@@ -877,11 +878,11 @@ Surface::readFromXPM(char** data)
   
   if(*current != '\0')
     {
-      debugN(17, cerr<<"[ERROR] " << endl;);
+      debugN(17, std::cerr<<"[ERROR] " << std::endl;);
       return false;
     }
 	   
-  debugN(17,cerr<<" [OK]"<< endl;);
+  debugN(17,std::cerr<<" [OK]"<< std::endl;);
 
   /* Now we know enough to build the surface.
    * We will always build a 32bit surface encoding the transparency 
@@ -899,7 +900,7 @@ Surface::readFromXPM(char** data)
   gatherPalette();
  
     
-  debugN(17, cerr << "parsing XPM data  ..."<< endl;);
+  debugN(17, std::cerr << "parsing XPM data  ..."<< std::endl;);
   Painter* painter = new Painter(this);
     
   for(int y = 0; y < h; y++)
@@ -908,7 +909,7 @@ Surface::readFromXPM(char** data)
       for(int x = 0; x < w; x++)
 	{
 	  // extract pixel code (cpp chars)
-	  code = string(current, cpp);
+	  code = std::string(current, cpp);
 	  //next pixel
 	  current+=cpp;
 	  
@@ -919,12 +920,12 @@ Surface::readFromXPM(char** data)
   bool success;
   if(*current != '\0')
     {
-      debugN(17, cerr<<"[ERROR] " << endl;);
+      debugN(17, std::cerr<<"[ERROR] " << std::endl;);
       success= false;
     }
   else
     {
-      debugN(17,cerr<<" [OK]"<< endl;);
+      debugN(17,std::cerr<<" [OK]"<< std::endl;);
       success = true;
     }
 
