@@ -1,18 +1,18 @@
 /*
   libuta - a C++ widget library based on SDL (Simple Direct Layer)
-  Copyright (C) 1999  Karsten Laux
+  Copyright (C) 1999-2002  Karsten Laux
 
   This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Library General Public
+  modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
-  version 2 of the License, or (at your option) any later version.
+  version 2.1 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Library General Public License for more details.
+  Lesser General Public License for more details.
   
-  You should have received a copy of the GNU Library General Public
+  You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, SA.
@@ -70,6 +70,7 @@ Widget::Widget(Widget* parent, int x, int y, int w, int h) :
   parentSurface_ = parent_->insertChild(this);
 
   setTransparency(true);
+
 }
 
 Widget::~Widget()
@@ -79,7 +80,7 @@ Widget::~Widget()
 //   if(!closed_)
 //     close();
 
-  debugN(17,cerr << name() << " has been hit for death." << endl;);
+  debugN(17,std::cerr << name() << " has been hit for death." << std::endl;);
 
   closed_ = true;
 
@@ -108,11 +109,13 @@ Widget::~Widget()
   if(surface_)
     delete surface_;
 
-  debugN(17,cerr << name() << " vanishes." << endl;);
+  debugN(17,std::cerr << name() << " vanishes." << std::endl;);
 
   // if we still own focus, release it before dying !
   if(haveFocus_)
     Focus::release(this);
+
+  deleted.emit();
 }
 
 void
@@ -201,7 +204,7 @@ Widget::setDisabledAlpha(unsigned char alpha)
 void
 Widget::blit()
 {
-  debugN(17,cerr << name() << " is blitted." << endl;);
+  debugN(17,std::cerr << name() << " is blitted." << std::endl;);
 
   if(!hidden_)
     {
@@ -209,7 +212,7 @@ Widget::blit()
       blit_complete(parentSurface_); 
 
       //blit the childs
-      list<Widget*>::iterator childItr_;
+      std::list<Widget*>::iterator childItr_;
       childItr_ = childs_.begin();
       while(childItr_ != childs_.end())
 	{
@@ -255,7 +258,7 @@ Widget::show()
   hideRequest_ = false;
   needsUpdate_ = true;
 
-  list<Widget*>::iterator childItr_;
+  std::list<Widget*>::iterator childItr_;
   childItr_ = childs_.begin();
   while(childItr_ != childs_.end())
     {
@@ -263,9 +266,10 @@ Widget::show()
       childItr_ ++;
     }  
   //force parent to update
-  if(parent_ != 0)
-    parent_->forceUpdate(1);
+  // if(parent_ != 0)
+  //  parent_->forceUpdate(1);
 
+  exposed.emit();
 }
 
 void
@@ -275,13 +279,15 @@ Widget::hide()
   hideRequest_ = true;
   needsUpdate_ = true;
 
-  list<Widget*>::iterator childItr_;
+  std::list<Widget*>::iterator childItr_;
   childItr_ = childs_.begin();
   while(childItr_ != childs_.end())
     {
       (*childItr_)->hide();
       childItr_ ++;
     }  
+
+  hidden.emit();
 }
 
 void
@@ -293,13 +299,15 @@ Widget::enable()
   disableRequest_ = false;
   needsUpdate_ = true;  
   
-  list<Widget*>::iterator childItr_;
+  std::list<Widget*>::iterator childItr_;
   childItr_ = childs_.begin();
   while(childItr_ != childs_.end())
     {
       (*childItr_)->enable();
       childItr_ ++;
     }  
+
+  enabled.emit();
 }
 
 void
@@ -308,13 +316,15 @@ Widget::disable()
   disableRequest_ = true;
   needsUpdate_ = true;
   
-  list<Widget*>::iterator childItr_;
+  std::list<Widget*>::iterator childItr_;
   childItr_ = childs_.begin();
   while(childItr_ != childs_.end())
     {
       (*childItr_)->disable();
       childItr_ ++;
     }  
+
+  disabled.emit();
 }
 
 void
@@ -324,6 +334,7 @@ Widget::close()
   //no chance...  every story has an end
   hidden_ = true;
   closed_ = true;
+
 }
 
 void
@@ -355,12 +366,12 @@ Widget::update()
     }
 	
   
-  debugN(17,cerr << name() << ".update()" << endl;);
+  debugN(17,std::cerr << name() << ".update()" << std::endl;);
 
   //remember update flag
   bool update_ = needsUpdate_;
   ///child list iterator.
-  list<Widget*>::iterator childItr_;
+  std::list<Widget*>::iterator childItr_;
   
   //create this child
   createChild();
@@ -371,7 +382,7 @@ Widget::update()
   needsReblit_ = false;
 
   childItr_ = childs_.begin();
-  list<Widget*> closeList;
+  std::list<Widget*> closeList;
 
   while(childItr_ != childs_.end())
     {
@@ -432,13 +443,13 @@ Widget::blit_dirty(Surface* target)
     {
       Rect dest;
       Rect src;
-      list<Rect>::iterator itr = dirtyRects_.begin();
+      std::list<Rect>::iterator itr = dirtyRects_.begin();
       
       while(itr != dirtyRects_.end())
 	{
 	  src = localCoord(*itr);
 	  dest = *itr;
-	  debugN(17,cerr <<name()<<" blits(" << src << " >>> "<<dest << endl;);
+	  debugN(17,std::cerr <<name()<<" blits(" << src << " >>> "<<dest << std::endl;);
 	  surface_->blit(target, dest, src);
 	  itr++;
 	}
@@ -454,7 +465,7 @@ Widget::createChild()
   if(needsUpdate_)
 #endif
     {
-      debugN(17,cerr << name() << ".createChild()" << endl;);
+      debugN(17,std::cerr << name() << ".createChild()" << std::endl;);
 
       if(!hidden_)
 	{
@@ -484,8 +495,8 @@ Widget::createChild()
       else
 	{
   	
-	  debugN(17,cerr << name() << " now wears the Cap of Invisibility."
-		<< endl;);
+	  debugN(17,std::cerr << name() << " now wears the Cap of Invisibility."
+		<< std::endl;);
 #ifndef DIRECT
 	  //create transparent bitmap
 	  surface_->clear();
@@ -494,13 +505,14 @@ Widget::createChild()
 	}
       needsUpdate_ = false;
     }
+
 }
 
 void
 Widget::create()
 {
 
-  debugN(17,cerr << name() << ".create() " << endl;);
+  debugN(17,std::cerr << name() << ".create() " << std::endl;);
 
   dirtyRects_.push_back(globalCoord(window_));
 
@@ -623,7 +635,7 @@ Widget::needsUpdate()
   //no ?, perhaps any child widget ?
 
   ///child list iterator.
-  list<Widget*>::iterator childItr_;
+  std::list<Widget*>::iterator childItr_;
   bool needed = false;
   childItr_ = childs_.begin();
   while(childItr_ != childs_.end())
@@ -659,7 +671,7 @@ Widget::insertChild(Widget* child)
   else
     bitmap = parentSurface_;
 
-  list<Widget*>::iterator itr = childs_.begin();
+  std::list<Widget*>::iterator itr = childs_.begin();
   while(itr != childs_.end())
     {
       if(*itr == child)
@@ -751,7 +763,7 @@ Widget::localCoord (Rect global)
 }
  
 void 
-Widget::addUpdateRects(list<Rect> &updateRects)
+Widget::addUpdateRects(std::list<Rect> &updateRects)
 {
   //this removes all elements in dirtyRects_
 
@@ -773,8 +785,23 @@ Widget::handleEvent(const Event* event)
   
   if(event->type() == Event::MOUSE)
     {
-      mouseFocus_ =  globalCoord(window_).
+      bool tmp =  globalCoord(window_).
 	contains(((MouseEvent *)event)->position());
+      
+      if(tmp && !mouseFocus_)
+	{
+	  mouseFocus_ = tmp;
+	  gainedMouse.emit();
+	}
+      else if(!tmp && mouseFocus_)
+	{
+	  mouseFocus_ = tmp;
+	  lostMouse.emit();
+	}
+      else
+	{
+	  mouseFocus_ = tmp;
+	}
     }
   
   //do we accept focus at all ?
@@ -796,8 +823,12 @@ Widget::handleEvent(const Event* event)
 	}
       
       //ok, try to obtain the focus...
-      if(grabFocus)
-	haveFocus_ = Focus::grab(this);
+      if(grabFocus && !haveFocus_)
+	{
+	  haveFocus_ = Focus::grab(this);
+	  gainedKeyboard.emit();
+	}
+      
     }
 
   //can we handle this event ?
@@ -808,7 +839,7 @@ Widget::handleEvent(const Event* event)
       //no ?, perhaps any child ?
 
       ///child list iterator.
-      list<Widget*>::iterator childItr_;
+      std::list<Widget*>::iterator childItr_;
       childItr_ = childs_.begin();
       while(childItr_ != childs_.end())
 	{
@@ -848,6 +879,8 @@ Widget::releaseFocus()
   //FIXME... sometime we may implemented locked focus ...
   haveFocus_ = false;
    
+  lostKeyboard.emit();
+
   needsUpdate_ = true;
 
   //ok,ok we are releasing the focus
@@ -860,8 +893,11 @@ Widget::grabFocus()
   haveFocus_ = Focus::grab(this);
   
   if(haveFocus_)
-    needsUpdate_ = true;
-  
+    {
+      needsUpdate_ = true;
+      gainedKeyboard.emit();
+    }
+
   return haveFocus_;
 }
 
